/*
 * paths.c
 *
 * This file is part of the msmtp package, a very simple programm that 
 * works as an "SMTP plugin" for Mutt and possibly other MUAs.
 *
 * Copyright (C) 2004
 * Martin Lambers <marlam@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   msmtp is released under the GPL with the additional exemption that
 *   compiling, linking, and/or using OpenSSL is allowed.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#ifdef WINDOWS
#include <windows.h>
#endif

#include "xmalloc.h"

#ifdef WINDOWS
#define PATH_SEP '\\'
#else /* UNIX or DJGPP */
#define PATH_SEP '/'
#endif


/*
 * get_prgname()
 *
 * Get the program name from an argv[0]-like string.
 * Returns a pointer to a static buffer.
 */

char *get_prgname(char *argv0)
{
    char *prgname;
    
    prgname = strrchr(argv0, PATH_SEP);
    if (!prgname)
    {
	prgname = argv0;
    }
    else
    {
	prgname++;
    }
    
    return prgname;
}


/*
 * get_homedir()
 *
 * Get the users home directory (or something similar, depending on the OS).
 * Returns an allocated string or NULL on error.
 */

char *get_homedir(void)
{
#ifdef WINDOWS

    BYTE home[MAX_PATH + 1];
    HKEY hkey;
    DWORD len;
    DWORD type;
    long l;
    
    l = RegOpenKeyEx(HKEY_CURRENT_USER,
	    "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 
	    0, KEY_READ, &hkey);
    if (l != ERROR_SUCCESS)
    {
	return NULL;
    }
    len = MAX_PATH;
    l = RegQueryValueEx(hkey, "AppData", NULL, &type, home, &len);
    if (l != ERROR_SUCCESS || len >= MAX_PATH)
    {
	RegCloseKey(hkey);
	return NULL;
    }
    RegCloseKey(hkey);
    return xstrdup((char *)home);

#else /* UNIX or DJGPP */
    
    char *home;
    if (!(home = getenv("HOME")))
    {
	return NULL;
    }
    return xstrdup(home);

#endif
}


/*
 * get_default_conffile()
 *
 * Get the default filename of the configuration file.
 * It consists of a user directory (see above) plus 'base'.
 * Returns an allocated string or NULL on error.
 */

char *get_default_conffile(char *base)
{
    char *conffile;
    size_t len;
    
    if (!(conffile = get_homedir()))
    {
	return NULL;
    }
    len = strlen(conffile);
    conffile = (char *)xrealloc(conffile, (len + strlen(base) + 2) * sizeof(char));
    if (len > 0 && conffile[len - 1] != PATH_SEP)
    {
	conffile[len++] = PATH_SEP;
    }
    strcpy(conffile + len, base);

    return conffile;
}


/*
 * tilde_expansion()
 *
 * If the first character of the given string is '~', it will be replaced by
 * the user's home directory. 
 * This function expects and returns an allocated string.
 * It never returns NULL: If there is no home directory, it doesn't change 
 * the filename. This way we don't depend on the availability of a home 
 * directory (->DOS).
 */

char *expand_tilde(char *filename)
{
    char *new_filename;
    
    if (filename[0] == '~' && (new_filename = get_homedir()))
    {
	new_filename = (char *)xrealloc(new_filename,
		(strlen(new_filename) + strlen(filename)) * sizeof(char));
	strcat(new_filename, filename + 1);
	free(filename);
	return new_filename;
    }
    else
    {
	return filename;
    }	
}
